/***************************************************************
 * Name:      datos.cpp
 * Purpose:   Code for Dialog nuevoProyecto
 * Author:    Salvador Pozo (salvapozo@gmail.com)
 * Created:   2024-07-20
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#include "datos.h"


bool datosProyecto::Minterm(minterm v) {
    for(size_t i=0; i<m.size(); i++) {
        if(m[i] == v) return true;
    }
    return false;
}

bool datosProyecto::Redundant(minterm v) {
    for(size_t i=0; i<r.size(); i++) {
        if(r[i] == v) return true;
    }
    return false;
}

wxString datosProyecto::ToBinary(int n) {
    wxString cad="";

    for(int i=0; i<nBits; i++) {
        if(n%2) cad = "1"+cad; else cad = "0"+cad;
        n/=2;
    }
    return cad;
}

bool datosProyecto::Cumple(int m) {
    wxString bits = ToBinary(m);
    bool cumple;

    for(size_t i=0; i<TablaFinal().size(); i++) {
        if(TablaFinal()[i].Esencial()) {
            cumple = true;
            for(int j=tablaFinal[i].Imp()->Tam()-1; cumple && j>=0; j--) {
                if(tablaFinal[i].Imp()->Bit()[j]!='-') cumple = tablaFinal[i].Imp()->Bit()[j] == bits[j];
//                if(TablaFinal()[i].Imp()->Bit()[j] == '1') salida << "I" << NBits()-j << " ";
//                if(TablaFinal()[i].Imp()->Bit()[j] == '0') salida << "I" << NBits()-j << "' ";
            }
            if(cumple) return true;
        }
    }
    return false;
}

void datosProyecto::Simplificar() {
    implicante::SetTam(nBits);

    grupo.clear();
    tablaFinal.clear();
    cuenta.clear();

    grupo.resize(nBits+1); // Crear grupos

    for(size_t i=0; i<m.size(); i++) {
        grupo[m[i].BitsActivos()].push_back(implicante(m[i]));
    }
    for(size_t i=0; i<r.size(); i++) {
        grupo[r[i].BitsActivos()].push_back(implicante(r[i]));
    }

    // Combinar grupos
    implicante imp;
    bool contiene;
    bool repetir;
    do {
        repetir = false;
        int nbit=0;
        // Combinar grupo de nbit con grupo de nbit+1
        while(nbit < nBits) {
            //std::cout << "Combina " << nbit << " con " << nbit+1 << std::endl;
            size_t s1 = grupo[nbit].size();
            size_t s2 = grupo[nbit+1].size();
            if(s1 && s2) {
                for(size_t i=0; i<s1; i++) {
                    for(size_t j=0; j<s2; j++) {
                        if(!grupo[nbit][i].Marca() && !grupo[nbit+1][j].Marca() && grupo[nbit][i]-grupo[nbit+1][j] == 1) {
                            imp = implicante(grupo[nbit][i], grupo[nbit+1][j]);
                            contiene=false;
                            for(size_t k=0; k<grupo[nbit].size() && !contiene; k++) contiene |= grupo[nbit][k] == imp;
                            if(!contiene) {
                                grupo[nbit].push_back(imp);
                                repetir = true;
                            }
                        }
                    }
                }
            }
            nbit++;
        }
        // Marcar los implicantes usados
        for(nbit=0; nbit<=nBits; nbit++) {
            size_t s1 = grupo[nbit].size();
            for(size_t i=0; i<s1; i++)
                if(grupo[nbit][i].Usado()) grupo[nbit][i].Marcar();
        }
    } while(repetir);

    bool terminado=false;
    size_t i, j;

    // Construir tabla final:
    for(int i=0; i<=nBits; i++) {
        size_t t=grupo[i].size();
        for(size_t j=0; j<t; j++) {
            if(!grupo[i][j].Marca()) {
                tablaFinal.push_back(implicanteprimo(&grupo[i][j], m));
            }
        }
    }


    // Seleccionar determinantes esenciales:
    // Los que contengan en exclusividad uno de los minterms.
    do {
        // Tabla de determinantes primos:
        do {
            repetir = false;
            for(size_t x=0; x<m.size(); x++) cuenta[m[x].Valor()] = 0;
            for(size_t i=0; i<tablaFinal.size(); i++) {
                if(!tablaFinal[i].Esencial()) tablaFinal[i].AjustaCuenta(cuenta);
            }
            // Buscar los determinantes con minters con cuenta igual a 1 (Si los hay)
            for(size_t x=0; x<m.size(); x++)
                if(cuenta[m[x].Valor()] == 1) { // Guarda bits:
                    repetir = true;
                    i=0;
                    while(!tablaFinal[i].Contiene(m[x]) && i<tablaFinal.size()) i++;
                    tablaFinal[i].MarcaEsencial();
                    for(j=0; j<tablaFinal.size(); j++) {
                        if(j != i) tablaFinal[j].EliminarMinterms(tablaFinal[i]);
                    }
                    tablaFinal[i].EliminarMinterms(tablaFinal[i]);
                }
        } while(repetir);
        terminado = true;
        for(j=0; terminado && j<tablaFinal.size(); j++) {
            if(!tablaFinal[j].Esencial() && tablaFinal[j].NMinterms()) terminado = false;
        }
        if(!terminado) {
            // Seleccionar determinantes prioritariamente por número de minterms, de mayor a menor
            // y por número de bits de menor a mayor (los ceros cuentas por 2)
            std::sort(tablaFinal.begin(), tablaFinal.end(), implicanteprimo::Comparar);

            // Seleccionamos el primero como esencial:
            for(j=1; j<tablaFinal.size(); j++) {
                tablaFinal[j].EliminarMinterms(tablaFinal[0]);
            }
            tablaFinal[0].EliminarMinterms(tablaFinal[0]);
            tablaFinal[0].MarcaEsencial();
            // Optimización opcional:
//            terminado = true;
//            for(j=0; terminado && j<data.TablaFinal().size(); j++) {
//                if(!data.TablaFinal()[j].Esencial() && data.TablaFinal()[j].NMinterms()) terminado = false;
//            }
        }
    } while(!terminado);

    SetResuelto(true);
}
