/***************************************************************
 * Name:      datos.h
 * Purpose:   Defines Dialog nuevoProyecto
 * Author:    Salvador Pozo (salvador@conclase.net)
 * Created:   2024-07-16
 * Copyright: Salvador Pozo (www.conclase.net)
 * License:
 **************************************************************/

#ifndef DATOSPROYECTO_H
#define DATOSPROYECTO_H

#include "wx/string.h"

#include "minterm.h"
#include "implicante.h"
#include "implicanteprimo.h"

class datosProyecto {
    public:
        datosProyecto(const wxString &n, int nb) : modificado(false), resuelto(false), nombreFichero(""), nombre(n), nBits(nb) {
            m.clear(); r.clear(); grupo.clear(); tablaFinal.clear(); cuenta.clear();
        }
        void SetModificado(bool m=true) { modificado=m; }
        bool Modificado() { return modificado; }
        void SetResuelto(bool r=true) { resuelto=r; }
        bool Resuelto() { return resuelto; }
        wxString& NombreFichero() { return nombreFichero; }
        wxString& Nombre() { return nombre; }
        int& NBits() { return nBits; }
        bool Minterm(minterm m);
        bool Redundant(minterm m);
        std::vector<minterm>& M() { return m; }
        std::vector<minterm>& R() { return r; }
        std::vector<std::vector<implicante>>& Grupo() { return grupo; }
        std::vector<implicanteprimo>& TablaFinal() { return tablaFinal; }
        std::map<int,int>& Cuenta() { return cuenta; }
        wxString ToBinary(int n);
        bool Cumple(int m);
        void Simplificar();

    private:
        bool modificado;
        bool resuelto;
        wxString nombreFichero;
        wxString nombre;
        int nBits;
        std::vector<minterm> m; /**< Lista de minterms de la función */
        std::vector<minterm> r; /**< Lista de minterms redundantes */
        std::vector<std::vector<implicante>> grupo;
        std::vector<implicanteprimo> tablaFinal;
        std::map<int,int> cuenta; // Cuenta de presencia de minterms en determinantes primos.
};

#endif
