/***************************************************************
 * Name:      nuevoproyecto.cpp
 * Purpose:   Code for Dialog editarMinterms
 * Author:    Salvador Pozo (salvapozo@gmail.com)
 * Created:   2024-07-16
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#include "editarMinterms.h"

BEGIN_EVENT_TABLE(editarMinterms, wxDialog)
    EVT_CLOSE(editarMinterms::OnClose)
    EVT_BUTTON(wxID_OK, editarMinterms::OnOk)
    EVT_LIST_ITEM_SELECTED(idMinterms, editarMinterms::OnSelect)
END_EVENT_TABLE()

editarMinterms::editarMinterms(wxWindow *wnd, datosProyecto &d)
    : wxDialog(wnd, -1, d.Nombre()), datos(d)
{
    wxBoxSizer *boxVertical;
    wxString cad;
    int nCombinaciones = 1 << datos.NBits();

    boxVertical = new wxBoxSizer(wxVERTICAL);

    boxVertical->Add(new wxStaticText(this, wxID_ANY, _T("&Minterms:")), wxSizerFlags().Border());
    ctrlMinterms = new wxListCtrl(this, idMinterms, wxDefaultPosition, wxSize(200, 400), wxLC_REPORT);
    ctrlMinterms->AppendColumn(_T("Minterm"));
    ctrlMinterms->AppendColumn(_T("Bits"));
    ctrlMinterms->AppendColumn(_T("Estado"));
    for(int i=0; i<nCombinaciones; i++) {
        cad = "";
        cad << i;
        ctrlMinterms->InsertItem(i, cad);
        ctrlMinterms->SetItem(i, 1, datos.ToBinary(i));
        if(datos.Minterm(i)) ctrlMinterms->SetItemData(i, 1);
        else if(datos.Redundant(i)) ctrlMinterms->SetItemData(i, 2);
        else ctrlMinterms->SetItemData(i, 0);
        switch(ctrlMinterms->GetItemData(i)) {
            case 0: ctrlMinterms->SetItem(i, 2, "-"); break;
            case 1: ctrlMinterms->SetItem(i, 2, "M"); break;
            case 2: ctrlMinterms->SetItem(i, 2, "R"); break;
        }
    }
    boxVertical->Add(ctrlMinterms, wxSizerFlags().Border());

    boxVertical->Add(CreateStdDialogButtonSizer(wxOK | wxCANCEL), wxSizerFlags().Border());

    SetSizerAndFit(boxVertical);
}

void editarMinterms::OnSelect(wxListEvent& event) {
    int id = event.GetIndex();
    ctrlMinterms->SetItemData(id, (ctrlMinterms->GetItemData(id)+1)%3);
    switch(ctrlMinterms->GetItemData(id)) {
        case 0: ctrlMinterms->SetItem(id, 2, "-"); break;
        case 1: ctrlMinterms->SetItem(id, 2, "M"); break;
        case 2: ctrlMinterms->SetItem(id, 2, "R"); break;
    }
}

void editarMinterms::OnClose(wxCloseEvent &event)
{
    Destroy();
}

void editarMinterms::OnOk(wxCommandEvent& event) {
    //Leer valores de lista:
    int nCombinaciones = 1 << datos.NBits();
    datos.M().clear();
    datos.R().clear();
    for(int id=0; id<nCombinaciones; id++) {
        switch(ctrlMinterms->GetItemData(id)) {
            case 1:
                datos.M().push_back(id);
                break;
            case 2:
                datos.R().push_back(id);
                break;
        }
    }
    Destroy();
}

