/***************************************************************
 * Name:      implicante.h
 * Purpose:   Defines for class implicante
 * Author:    Salvador Pozo (salvapozo@gmail.com)
 * Created:   2024-07-16
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#ifndef IMPLICANTE_H
#define IMPLICANTE_H

#include <list>
#include <string>
#include "minterm.h"

// Clase para encapsular implicantes
class implicante {
public:
    // Constructor a partir de un minter v
    implicante(minterm m=0) : marca(false), usado(false) {
        int mask = 1;
        lminterms.push_back(m.Valor());
        for(int i=0; i<tam; i++) {
            bits.insert(0, 1, (m.Valor() & mask) ? '1' :'0');
            mask <<= 1;
        }
    }
    // Constructor a partir de dos implicantes i1 e i2
    implicante(implicante &i1, implicante &i2 ) : marca(false), usado(false) {
        for(minterm n : i1.lminterms) lminterms.push_back(n);
        for(minterm n : i2.lminterms) lminterms.push_back(n);
        //lminterms.sort(); // No es necesario
        for(int i=0; i<tam; i++) {
            bits.push_back((i1.bits[i] == i2.bits[i]) ? i1.bits[i] :'-');
        }
        i1.usado = i2.usado = true;
    }
    static void SetTam(int t) { tam = t; }
    bool operator==(implicante &i) {
        return bits ==i.bits;
    }
    // Calcula el número de bits diferentes en dos implicantes
    int operator-(implicante &i2) {
        int d = 0;
        for(int i=0; i<tam; i++) {
            if(bits[i] != i2.bits[i]) d++;
        }
        return d;
    }
    // Interfaz
    std::string &Bit() { return bits; }
    int Tam() const { return tam; }
    std::list<minterm> &Minterm() { return lminterms; }
    bool Marca() const { return marca; }
    bool Usado() const { return usado; }
    void Marcar(bool m = true ) { marca = m; }
private:
    static int tam; // Tamaño del implicante = número de entradas
    std::string bits; // bits del implicante
    std::list<minterm> lminterms; // Lista de minterms que componen el implicante
    bool marca; // true si el implicante ha sido usado en una combinación
    bool usado; // variable auxiliar
};

#endif // header guard

