/***************************************************************
 * Name:      implicanteprimo.cpp
 * Purpose:   Code for class implicanteprimo
 * Author:    Salvador Pozo (salvapozo@gmail.com)
 * Created:   2024-07-16
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#include "implicanteprimo.h"

implicanteprimo::implicanteprimo(implicante *i, std::vector<minterm> &m) : imp(i), esencial(false), nMinterms(0), nOperaciones(0) {
    for(size_t n=0; n<m.size(); n++) presente[m[n].Valor()] = false;
    for(minterm n : imp->Minterm())
        if(presente.find(n.Valor())!= presente.end()) {
            presente[n.Valor()] = true;
            nMinterms++;
        }
    for(int i=0; i<imp->Tam(); i++) {
        if(imp->Bit()[i] == '1') nOperaciones++;
        if(imp->Bit()[i] == '0') nOperaciones+=2;
    }
}
implicanteprimo::implicanteprimo(const implicanteprimo &t) : imp(t.imp), esencial(t.esencial), nMinterms(t.nMinterms), nOperaciones(t.nOperaciones) {
    for (auto it = t.presente.begin(); it != t.presente.end(); ++it) {
        presente[it->first] = it->second;
    }
}

implicanteprimo &implicanteprimo::operator=(const implicanteprimo &t) {
    if(&t != this) {
        imp = t.imp;
        esencial = t.esencial;
        nMinterms = t.nMinterms;
        nOperaciones = t.nOperaciones;
        presente.clear();
        for (auto it = t.presente.begin(); it != t.presente.end(); ++it) {
            presente[it->first] = it->second;
        }
    }
    return *this;
}

bool implicanteprimo::Comparar(implicanteprimo &a, implicanteprimo &b) {
    if(a.nMinterms > b.nMinterms) return true;
    if(a.nMinterms == b.nMinterms && a.nOperaciones < b.nOperaciones) return true;
    return false;
}

void implicanteprimo::AjustaCuenta(std::map<int,int> &cuenta) {
    for (auto it = presente.begin(); it != presente.end(); ++it) {
        if(it->second) cuenta[it->first]++;
    }
}

void implicanteprimo::EliminarMinterms(implicanteprimo &t) {
    for (auto it = t.presente.begin(); it != t.presente.end(); ++it) {
        if(it->second && presente[it->first]) {
            presente[it->first] = false;
            nMinterms--;
        }
    }
}

bool implicanteprimo::Contiene(minterm &m) const {
    for(minterm n : imp->Minterm() ) if(n == m) return true;
    return false;
}
