/***************************************************************
 * Name:      implicanteprimo.h
 * Purpose:   Defines for class implicanteprimo
 * Author:    Salvador Pozo (salvapozo@gmail.com)
 * Created:   2024-07-16
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#ifndef IMPLICANTEPRIMO_H
#define IMPLICANTEPRIMO_H

#include <iostream>
#include <iomanip>

#include <vector>
#include <map>
#include "implicante.h"

// Clase para encapsular implicantes primos
class implicanteprimo {
public:
    // Constructor
    implicanteprimo(implicante *i, std::vector<minterm> &m);
    // Constructor copia, necesario para aplicar el algoritmo sort
    implicanteprimo(const implicanteprimo &t);
    // Operador de asignación, necesario para aplicar el algoritmo sort
    implicanteprimo &operator=(const implicanteprimo &t);
    // Función estática de comparación usada para ordenar vectores de implicantes primos
    static bool Comparar(implicanteprimo &a, implicanteprimo &b);
    // Incrementa los elementos del mapa 'cuenta' con los minterms de este implicante implicanteprimo
    void AjustaCuenta(std::map<int,int> &cuenta);
    // Elimina los minterms de este implicante presentes en el parámetro t
    void EliminarMinterms(implicanteprimo &t);
    // Devuelve true si el implicante contiene el minterm del parámetro
    bool Contiene(minterm &m) const;
    // Interfaz
    bool Esencial() { return esencial; }
    void MarcaEsencial() { esencial = true; }
    int NMinterms() const { return nMinterms; }
    int NOperaciones() const { return nOperaciones; }
    std::map<int,bool> &Presente() { return presente; }
    implicante *Imp() { return imp; }

private:
    implicante *imp; // Puntero al implicante dentro del vector de implicantes
    std::map<int,bool> presente; // Indica si el implicante primo contiene o no los minterms necesarios
    bool esencial; // Implicante primo marcado como esencial
    int nMinterms; // Usado para ordenar, número de minterms que contiene el implicante
    int nOperaciones; // Usado para ordenar, número de operaciones del implicante primo
};

#endif // header guard

