/***************************************************************
 * Name:      wx_Quine_McCluskeyMain.cpp
 * Purpose:   Code for Application Frame
 * Author:    Salvador Pozo (salvador@conclase.net)
 * Created:   2024-07-15
 * Copyright: Salvador Pozo (https://conclase.net)
 * License:
 **************************************************************/

#ifdef WX_PRECOMP
#include "wx_pch.h"
#endif

#include "wx_Quine_McCluskeyMain.h"
#include "nuevoproyecto.h"
#include "editarMinterms.h"
#include "verificar.h"

//helper functions
enum wxbuildinfoformat {
    short_f, long_f };

wxString wxbuildinfo(wxbuildinfoformat format)
{
    wxString wxbuild(wxVERSION_STRING);

#if defined(__WXMSW__)
        wxbuild << _T("-Windows");
#elif defined(__WXMAC__)
        wxbuild << _T("-Mac");
#elif defined(__UNIX__)
        wxbuild << _T("-Linux");
#endif

#if wxUSE_UNICODE
        wxbuild << _T("-compilación Unicode");
#else
        wxbuild << _T("-compilación ANSI");
#endif // wxUSE_UNICODE

    if (format == long_f ) {
        wxbuild << _T("\n\nSimplificación de funciones booleanas\n");
        wxbuild << _T("Mediante el método de Quine McCluskey\n\n");
        wxbuild << _T("2024 Con Clase\n");
        wxbuild << _T("Salvador Pozo\n\n");
    }
    return wxbuild;
}


BEGIN_EVENT_TABLE(wx_Quine_McCluskeyFrame, wxFrame)
    EVT_CLOSE(wx_Quine_McCluskeyFrame::OnClose)
    EVT_SIZE(wx_Quine_McCluskeyFrame::OnSize)
    EVT_MENU(idMenuQuit, wx_Quine_McCluskeyFrame::OnQuit)
    EVT_MENU(idMenuAbout, wx_Quine_McCluskeyFrame::OnAbout)
    EVT_MENU(idMenuNew, wx_Quine_McCluskeyFrame::OnNuevoProyecto)
    EVT_MENU(idMenuEdit, wx_Quine_McCluskeyFrame::OnEditarProyecto)
    EVT_MENU(idMenuDefine, wx_Quine_McCluskeyFrame::OnDefinirProyecto)
    EVT_MENU(idMenuSave, wx_Quine_McCluskeyFrame::OnGuardarProyecto)
    EVT_MENU(idMenuSaveAs, wx_Quine_McCluskeyFrame::OnGuardarComoProyecto)
    EVT_MENU(idMenuLoad, wx_Quine_McCluskeyFrame::OnCargarProyecto)
    EVT_MENU(idMenuSimplify, wx_Quine_McCluskeyFrame::OnSimplify)
    EVT_MENU(idMenuVerify, wx_Quine_McCluskeyFrame::OnVerify)
END_EVENT_TABLE()

wx_Quine_McCluskeyFrame::wx_Quine_McCluskeyFrame(wxFrame *frame, const wxString &title)
    : wxFrame(frame, -1, title),
    data(_T("Nombre"),4)
{
    // create a menu bar
    wxMenuBar* mbar = new wxMenuBar();
    fileMenu = new wxMenu(_T(""));
    fileMenu->Append(idMenuNew, _("&Nuevo...\tAlt-F2"), _T("Nuevo proyecto"));
    fileMenu->Append(idMenuEdit, _("&Editar...\tAlt-F3"), _T("Modificar proyecto"));
    fileMenu->Append(idMenuDefine, _("&Definir...\tAlt-F4"), _T("Definir minterms"));
    fileMenu->Append(-1, _T(""), _T(""), wxITEM_SEPARATOR);
    fileMenu->Append(idMenuSimplify, _("&Simplificar...\tAlt-F5"), _T("Simplificar función"));
    fileMenu->Append(idMenuVerify, _("&Verificar...\tAlt-F6"), _T("Verificar función"));
    fileMenu->Append(-1, _T(""), _T(""), wxITEM_SEPARATOR);
    fileMenu->Append(idMenuSave, _("&Guardar\tCtrl-S"), _T("Guardar"));
    fileMenu->Append(idMenuSaveAs, _("Guardar &como..."), _T("Guardar como"));
    fileMenu->Append(idMenuLoad, _("Car&gar...\tCtrl-L"), _T("Cargar"));
    fileMenu->Append(-1, _T(""), _T(""), wxITEM_SEPARATOR);
    fileMenu->Append(idMenuQuit, _("&Salir\tAlt-F4"), _T("Salir de la aplicación"));
    mbar->Append(fileMenu, _("&Fichero"));

    wxMenu* helpMenu = new wxMenu(_T(""));
    helpMenu->Append(idMenuAbout, _("S&obre...\tF1"), _T("Mostrar información sobre esta aplicación"));
    mbar->Append(helpMenu, _("&Ayuda"));

    SetMenuBar(mbar);
    fileMenu->Enable(idMenuSave, false);
    fileMenu->Enable(idMenuVerify, false);

    // create a status bar with some information about the used wxWidgets version
    int widths[] = {-1, -2, 8};
    CreateStatusBar(3);
    SetStatusWidths(3, widths);
    SetStatusText(_("Hola"),0);
    SetStatusText(wxbuildinfo(short_f), 1);
    wxRect rect;
    GetStatusBar()->GetFieldRect(2, rect);
    bm = new wxStaticBitmap(GetStatusBar(), wxID_ANY, wxBITMAP_PNG(verde), wxPoint(rect.x, rect.y));
}

wx_Quine_McCluskeyFrame::~wx_Quine_McCluskeyFrame()
{
}

void wx_Quine_McCluskeyFrame::OnClose(wxCloseEvent &event) {
    if(data.Modificado())
        if(wxMessageBox(_T("El proyecto ha sido modificado,\n¿Quieres cerrar la aplicación sin guardalo?"), data.Nombre(), wxYES_NO) == wxNO)
            return;
    Destroy();
}

void wx_Quine_McCluskeyFrame::OnQuit(wxCommandEvent &event) {
    if(data.Modificado())
        if(wxMessageBox(_T("El proyecto ha sido modificado,\n¿Quieres cerrar la aplicación sin guardalo?"), data.Nombre(), wxYES_NO) == wxNO)
            return;
    Destroy();
}

void wx_Quine_McCluskeyFrame::OnSize(wxSizeEvent& event) {
    if(!GetStatusBar()) return;
    ActualizarStatusBar();
}

void wx_Quine_McCluskeyFrame::ActualizarStatusBar() {
    wxRect rect;

    GetStatusBar()->GetFieldRect(2, rect);
    delete bm;
    if(data.Modificado())
        bm = new wxStaticBitmap(GetStatusBar(), wxID_ANY, wxBITMAP_PNG(rojo), wxPoint(rect.x, rect.y));
    else
        bm = new wxStaticBitmap(GetStatusBar(), wxID_ANY, wxBITMAP_PNG(verde), wxPoint(rect.x, rect.y));
}

void wx_Quine_McCluskeyFrame::OnAbout(wxCommandEvent &event)
{
    wxString msg = wxbuildinfo(long_f);
    wxMessageBox(msg, _("Quine McCluskey wx"));
}

void wx_Quine_McCluskeyFrame::OnNuevoProyecto(wxCommandEvent& event)
{
    data.SetModificado(true);
    ActualizarStatusBar();
    data.Nombre() = "Nombre";
    data.NBits() = 4;
    data.M().clear();
    data.R().clear();
    data.Grupo().clear();
    data.TablaFinal().clear();
    data.Cuenta().clear();
    nuevoProyecto *dlg = new nuevoProyecto(this, data, _T("Nuevo proyecto"));
    dlg->ShowModal();
    SetStatusText(data.Nombre(),0);
    fileMenu->Enable(idMenuSave, false);
    data.SetResuelto(false);
    fileMenu->Enable(idMenuVerify, false);
}

void wx_Quine_McCluskeyFrame::OnEditarProyecto(wxCommandEvent& event)
{
    nuevoProyecto *dlg = new nuevoProyecto(this, data, _T("Editar proyecto"));
    if(dlg->ShowModal() == wxID_CANCEL) return;
    data.SetModificado(true);
    data.SetResuelto(false);
    fileMenu->Enable(idMenuVerify, false);
    ActualizarStatusBar();
    SetStatusText(data.Nombre(),0);
}

void wx_Quine_McCluskeyFrame::OnDefinirProyecto(wxCommandEvent& event)
{
    editarMinterms *dlg = new editarMinterms(this, data);
    if(dlg->ShowModal() == wxID_CANCEL) return;
    data.SetModificado(true);
    data.SetResuelto(false);
    fileMenu->Enable(idMenuVerify, false);
    ActualizarStatusBar();
}

void wx_Quine_McCluskeyFrame::Save() {
    xmlDocPtr doc;
    xmlNodePtr nodo_raiz, nodoMinterms, nodoRedundantes;
    xmlNodePtr comentario;
    const xmlChar *version = BAD_CAST "1.0";
    xmlChar documento[] = "Quine_McCluskey";
    xmlChar nombre[] = "nombre";
    xmlChar nbits[] = "nbits";
    xmlChar minterms[] = "minterms";
    xmlChar term[] = "term";
    xmlChar redundantes[] = "redundantes";
    xmlChar *cad;
    wxString cad2;

    LIBXML_TEST_VERSION;

    // Crear el documento:
    doc = xmlNewDoc(version);

    // Insertar un comentario para el documento:
    comentario = xmlNewDocComment(doc, BAD_CAST "Simplificación de funciones booleanas");
    xmlAddChild(xmlNodePtr(doc), comentario);

    // Crear el nodo principal:
    nodo_raiz = xmlNewDocNode(doc, NULL, documento, NULL);
    xmlDocSetRootElement(doc, nodo_raiz);

    // Crear un nodo para el nombre:
    cad = BAD_CAST data.Nombre().ToUTF8().data();
    xmlNewChild(nodo_raiz, NULL, nombre, cad);

    // Crear un nodo para el número de bits:
    cad2 << data.NBits();
    cad = BAD_CAST cad2.ToUTF8().data();
    xmlNewChild(nodo_raiz, NULL, nbits, cad);

    nodoMinterms = xmlNewChild(nodo_raiz, NULL, minterms, NULL);
    for(size_t i = 0; i < data.M().size(); i++) {
        cad2 = "";
        cad2 << data.M()[i].Valor();
        cad = BAD_CAST cad2.ToUTF8().data();
        xmlNewChild(nodoMinterms, NULL, term, cad);
    }

    nodoRedundantes = xmlNewChild(nodo_raiz, NULL, redundantes, NULL);
    for(size_t i = 0; i < data.R().size(); i++) {
        cad2 = "";
        cad2 << data.R()[i].Valor();
        cad = BAD_CAST cad2.ToUTF8().data();
        xmlNewChild(nodoRedundantes, NULL, term, cad);
    }

    // Guardar en fichero:
    xmlSaveFileEnc(data.NombreFichero(), doc, "UTF8");

    xmlFreeDoc(doc);
    xmlMemoryDump();
}

void wx_Quine_McCluskeyFrame::Load(wxString file) {
    xmlDocPtr doc;
    xmlXPathContextPtr xpathCtx;
    bool valido=false;
    wxString cad;

    xmlInitParser();
    LIBXML_TEST_VERSION

    doc = xmlReadFile(file, NULL, XML_PARSE_NOBLANKS);

    xpathCtx = xmlXPathNewContext(doc);

    // Verificar si el fichero contiene una función válida para esta aplicación:
    if(ExisteNodo(xpathCtx, BAD_CAST "/Quine_McCluskey/nombre") &&
       ExisteNodo(xpathCtx, BAD_CAST "/Quine_McCluskey/nbits") &&
       ExisteNodo(xpathCtx, BAD_CAST "/Quine_McCluskey/minterms") &&
       ExisteNodo(xpathCtx, BAD_CAST "/Quine_McCluskey/redundantes")) {
        valido=true;
    }

    // Hacer esto solo si el fichero contiene una función Quine/McCluskey
    if(valido) {
        data.NombreFichero() = file;
        cad = LeeNodoString(xpathCtx, BAD_CAST "/Quine_McCluskey/nombre");
        data.Nombre() = cad; //.wc_str();
        data.NBits() = LeeNodoInt(xpathCtx, BAD_CAST "/Quine_McCluskey/nbits");
        LeeMinterms(xpathCtx, BAD_CAST "/Quine_McCluskey/minterms/*");
        LeeRedundantes(xpathCtx, BAD_CAST "/Quine_McCluskey/redundantes/*");
        fileMenu->Enable(idMenuSave, true);
    }

    /* Cleanup */
    xmlXPathFreeContext(xpathCtx);
    xmlFreeDoc(doc);
    xmlCleanupParser();

}

bool wx_Quine_McCluskeyFrame::ExisteNodo(xmlXPathContextPtr xpathCtx, const xmlChar *ruta) {
    xmlXPathObjectPtr xpathObj;
    int size;

    xpathObj = xmlXPathEvalExpression(ruta, xpathCtx);
    size = (xpathObj->nodesetval) ? xpathObj->nodesetval->nodeNr : 0;
    xmlXPathFreeObject(xpathObj);
    return size==1;
}

wxString wx_Quine_McCluskeyFrame::LeeNodoString(xmlXPathContextPtr xpathCtx, const xmlChar *ruta) {
    xmlXPathObjectPtr xpathObj;
    xmlNodePtr cur;
    wxString retval;

    xpathObj = xmlXPathEvalExpression(ruta, xpathCtx);
    cur = xpathObj->nodesetval->nodeTab[0];
    retval = wxString::FromUTF8(reinterpret_cast<char*>(cur->children->content));
    xmlXPathFreeObject(xpathObj);
    return retval;
}

int wx_Quine_McCluskeyFrame::LeeNodoInt(xmlXPathContextPtr xpathCtx, const xmlChar *ruta) {
    xmlXPathObjectPtr xpathObj;
    xmlNodePtr cur;
    int retval;

    xpathObj = xmlXPathEvalExpression(ruta, xpathCtx);
    cur = xpathObj->nodesetval->nodeTab[0];
    wxString(cur->children->content).ToInt(&retval); //std::atoi(reinterpret_cast<const char*>(cur->children->content));
    xmlXPathFreeObject(xpathObj);
    return retval;
}

void wx_Quine_McCluskeyFrame::LeeMinterms(xmlXPathContextPtr xpathCtx, const xmlChar *ruta) {
    xmlXPathObjectPtr xpathObj;
    xmlNodePtr cur;
    int size, i;

    xpathObj = xmlXPathEvalExpression(ruta, xpathCtx);
    data.M().clear();
    if(!xpathObj) return;
    size = (xpathObj->nodesetval) ? xpathObj->nodesetval->nodeNr : 0;
    for(i = 0; i < size; ++i) {
        cur = xpathObj->nodesetval->nodeTab[i];
        data.M().push_back(std::atoi(reinterpret_cast<const char*>(cur->children->content)));
    }
    xmlXPathFreeObject(xpathObj);
}

void wx_Quine_McCluskeyFrame::LeeRedundantes(xmlXPathContextPtr xpathCtx, const xmlChar *ruta) {
    xmlXPathObjectPtr xpathObj;
    xmlNodePtr cur;
    int size, i;

    xpathObj = xmlXPathEvalExpression(ruta, xpathCtx);
    data.R().clear();
    if(!xpathObj) return;
    size = (xpathObj->nodesetval) ? xpathObj->nodesetval->nodeNr : 0;
    for(i = 0; i < size; ++i) {
        cur = xpathObj->nodesetval->nodeTab[i];
        data.R().push_back(std::atoi(reinterpret_cast<const char*>(cur->children->content)));
    }
    xmlXPathFreeObject(xpathObj);
}

void wx_Quine_McCluskeyFrame::OnGuardarProyecto(wxCommandEvent& event){
    Save();
    data.SetModificado(false);
    ActualizarStatusBar();
}

void wx_Quine_McCluskeyFrame::OnGuardarComoProyecto(wxCommandEvent& event) {
    wxFileDialog dlg(this, _T("Guardar como"), "", "", _T("ficheros xml (*.xml)|*.xml"), wxFD_SAVE|wxFD_OVERWRITE_PROMPT);
    if(dlg.ShowModal() == wxID_CANCEL) return;
    data.NombreFichero() = dlg.GetPath();
    fileMenu->Enable(idMenuSave, true);
    Save();
    data.SetModificado(false);
    ActualizarStatusBar();
}

void wx_Quine_McCluskeyFrame::OnCargarProyecto(wxCommandEvent& event) {
    if(data.Modificado())
        if(wxMessageBox(_T("El proyecto ha sido modificado,\n¿Quieres cargar un nuevo proyecto sin guardalo?"), data.Nombre(), wxYES_NO) == wxNO)
            return;
    wxFileDialog dlg(this, _T("Cargar"), "", "", _T("ficheros xml (*.xml)|*.xml"), wxFD_OPEN|wxFD_FILE_MUST_EXIST);
    if(dlg.ShowModal() == wxID_CANCEL) return;
    Load(dlg.GetPath());
    SetStatusText(data.Nombre(),0);
    data.SetModificado(false);
    data.SetResuelto(false);
    fileMenu->Enable(idMenuVerify, false);

    ActualizarStatusBar();
}

void wx_Quine_McCluskeyFrame::OnSimplify(wxCommandEvent& event) {
    data.Simplificar();
    fileMenu->Enable(idMenuVerify, true);

    wxString salida(_T("Resultado\n"));
    bool primero=true;
    salida << "S = ";
    for(size_t i=0; i<data.TablaFinal().size(); i++) {
        if(data.TablaFinal()[i].Esencial()) {
            if(!primero) { salida << "+ "; }
            else primero=false;
            for(int j=data.TablaFinal()[i].Imp()->Tam()-1; j>=0; j--) {
                if(data.TablaFinal()[i].Imp()->Bit()[j] == '1') salida << "I" << data.NBits()-j << " ";
                if(data.TablaFinal()[i].Imp()->Bit()[j] == '0') salida << "I" << data.NBits()-j << "' ";
            }
        }
    }
    wxMessageBox(salida, "Salida");

}

void wx_Quine_McCluskeyFrame::OnVerify(wxCommandEvent& event) {
    verificarProyecto *dlg = new verificarProyecto(this, data);
    dlg->ShowModal();
}
